require "asm"
require "asm/util"
require "fileutils"
require "tempfile"
require "erb"

module ASM
  # Utility module for building custom iPXE ISO images
  module IpxeBuilder
    IPXE_DIR = ASM.config.ipxe_src_dir
    LOCK = Mutex.new

    def self.ipxe_dir
      IPXE_DIR
    end

    def self.ipxe_src_dir
      File.join(ipxe_dir, "src")
    end

    def self.path_to_template(template)
      File.read(File.expand_path(File.join(File.dirname(__FILE__), "..", "..", "templates", template)))
    end

    def self.render_bootstrap(network_config, n_nics)
      ERB.new(path_to_template("bootstrap.ipxe.erb"), nil, "-").result(binding)
    end

    # Generates a bootstrap.ipxe file for specified network configuration
    #
    # @param network_config [ASM::NetworkConfiguration] the configuration. Must have had {ASM::NetworkConfiguration#add_nics!} called on it.
    # @param n_nics [FixNum] the total number of network devices on the target server
    # @param path [String] the file path to save the resulting bootstrap script to
    def self.generate_bootstrap(network_config, n_nics, path)
      File.write(path, render_bootstrap(network_config, n_nics))
    end

    # Build a custom iPXE ISO image targeted to the specified network configuration
    #
    # The custom ISO image will contain a bootstrap.ipxe file tailored to the
    # provided network configuration object. On boot with the ISO, ethernet devices
    # corresponding to the PXE partitions in the network configuration will be configured with
    # either static or DHCP networking as specified by the PXE network. Each ethernet
    # device will be tried sequentially, so if iPXE chain-loading fails on one
    # device the boot will be retried with subsequent devices.
    #
    # @param network_config [ASM::NetworkConfiguration] the configuration. Must have had {ASM::NetworkConfiguration#add_nics!} called on it.
    # @param n_nics [FixNum] the total number of network devices on the target server
    # @param dest_filename [String] the file path to save the resulting ISO
    # @raise [StandardError] if a failure occurs while building the ISO
    def self.build(network_config, n_nics, dest_filename)
      bootstrap_file = nil
      LOCK.synchronize do
        bootstrap_file = Tempfile.new("bootstrap.ipxe")
        generate_bootstrap(network_config, n_nics, bootstrap_file.path)
        ret = ASM::Util.run_command("make", "-C", ipxe_src_dir,
                                    "bin/ipxe.iso", "EMBED=%s" % bootstrap_file.path)
        raise("iPXE ISO build failed: %s" % ret.to_s) unless ret.exit_status == 0
        FileUtils.mv(File.join(ipxe_src_dir, "bin", "ipxe.iso"), dest_filename)
      end
      nil
    ensure
      bootstrap_file.unlink if bootstrap_file
    end
  end
end
