require "asm/provider/switch/force10/base"

module ASM
  class Provider
    class Switch
      class Force10
        # Manage Interface VLAN membership for Dell Force10 blade switches
        #
        # These switches unlike their TOR cousins are configured using a simple
        # 1 resource per interface method so this class mainly builds a temporary
        # map of every interface and all its VLAN memberships via {#configure_interface_vlan}
        # and then turns that into an appropriate puppet resource using {#to_puppet}
        class BladeIoa < Base
          def configure_iom_mode!(vlt, pmux, ethernet_mode)
            raise("iom mode resource is already created in this instance for switch %s" % type.puppet_certname) if port_resources["ioa_mode"]

            if vlt
              port_resources["ioa_mode"] = vlt_ioa_mode_resource
            elsif pmux
              port_resources["ioa_mode"] = pmux_ioa_mode_resource(ethernet_mode)
            end

            if port_resources["ioa_mode"]
              key, resource = port_resources["ioa_mode"].first
              resource["require"] = sequence if sequence
              @sequence = "Ioa_mode[%s]" % key
            end

            nil
          end

          # (see Provider::Switch::Force10#ioa_interface_resource)
          def ioa_interface_resource(interface, tagged_vlans, untagged_vlans)
            if has_resource?("ioa_interface", interface)
              raise("Ioa_interface[%s] is already being managed on %s" % [interface, type.puppet_certname])
            end

            port_resources["ioa_interface"] ||= {}
            port_resources["ioa_interface"][interface] = {}

            port_resources["ioa_interface"][interface]["vlan_tagged"] = tagged_vlans.join(",") unless tagged_vlans.empty?
            port_resources["ioa_interface"][interface]["vlan_untagged"] = untagged_vlans.join(",") unless untagged_vlans.empty?
            port_resources["ioa_interface"][interface]["switchport"] = true
            port_resources["ioa_interface"][interface]["portmode"] = "hybrid"
            port_resources["ioa_interface"][interface]["require"] = sequence if sequence

            @sequence = "Ioa_interface[%s]" % interface

            nil
          end

          def vlt_ioa_mode_resource
            {
              "vlt" => {
                "iom_mode" => "vlt",
                "ioa_ethernet_mode" => "true",
                "ensure" => "present"
              }
            }
          end

          def pmux_ioa_mode_resource(ethernet_mode)
            {
              "pmux" => {
                "iom_mode" => "pmux",
                "ensure" => "present",
                "ioa_ethernet_mode" => ethernet_mode.to_s
              }
            }
          end

          # Creates interfaces resources for consumption by puppet
          #
          # @return [Hash]
          def to_puppet
            if port_resources["ioa_interface"]
              port_resources["ioa_interface"].keys.each do |interface|
                # the puppet type does not accept arrays for interfaces at the moment, so join them as csv and delete empty ones
                resource = port_resources["ioa_interface"][interface]

                ["vlan_tagged", "vlan_untagged"].each do |prop|
                  next unless resource[prop].is_a?(Array)
                  resource[prop] = resource[prop].sort.uniq.join(",")
                end
              end
            end

            port_resources
          end

          # Prepares the final internal state for a given action
          #
          # This should be called before {#to_puppet} to construct
          # the state based on interface information created using
          # {#configure_interface_vlan}
          #
          # The return value indicates if there were any interfaces to
          # configure for the action, if it's false there's no point
          # in calling process for the type as there's nothing to process
          #
          # @param [:add, :remove] action
          # @return [Boolean] if any interfaces were found for the action
          def prepare(action)
            reset!
            validate_vlans!
            populate_interface_resources(action)
            disable_autolag unless interface_map.empty?
            !port_resources.empty?
          end

          # Find all the interfaces for a certain action and create interface resources
          #
          # Interfaces are made using {#configure_interface_vlan} and are
          # marked as add or remove.  This finds all previously made interfaces
          # for a given action and populates the {#port_resources} hash with the
          # right resources
          #
          # @param [:add, :remove] action
          # @return [void]
          def populate_interface_resources(action)
            interface_map.each do |interface|
              next unless interface[:action] == action

              port_resources["ioa_interface"] ||= {}

              unless resource = port_resources["ioa_interface"][interface[:interface]]
                resource = port_resources["ioa_interface"][interface[:interface]] = {
                  "vlan_tagged" => [],
                  "vlan_untagged" => []
                }

                resource["require"] = sequence if sequence

                @sequence = "Ioa_interface[%s]" % interface[:interface]
              end

              if interface[:tagged]
                resource["vlan_tagged"] << interface[:vlan]
              else
                resource["vlan_untagged"] << interface[:vlan]
              end

              resource["switchport"] = true
              resource["portmode"] = "hybrid"
            end
          end

          def disable_autolag
            port_resources["ioa_autolag"] = {
              "ioa_autolag" => {
                "ensure" => "absent"
              }
            }
          end

          # Reset each port in the switch to a default state
          #
          # @note this will populate the resources without running puppet
          # @return [void]
          def initialize_ports!
            if provider.model =~ /Aggregator/
              port_names.each do |port|
                configure_interface_vlan(port, "1", false, true)
                configure_interface_vlan(port, "1", true, true)
              end

              populate_interface_resources(:remove)
            end
          end

          # Produce a list of port names for a certain switch
          #
          # @note only returned TE names not FC and only for a single unit
          # @return [Array<String>] array of port names
          def port_names
            (1..port_count).map {|i| "Te 0/%s" % i}
          end
        end
      end
    end
  end
end
