require "asm/provider/switch/force10/base"

module ASM
  class Provider
    class Switch
      class Force10
        # Manage VLAN membership for Dell Force10 top of rack switches
        #
        # These switches are managed using at least 2 resources for every
        # interface.
        #
        # First a force10_interface resource needs to exist for every
        # interface under management then for every VLAN a asm::force10
        # resource exist that lists all the tagged and untagged interfaces
        # that belong to it.
        #
        # To achieve this the {#configure_interface_vlan} method is called
        # for every VLAN - can be called many times per interface
        #
        # Once that is done the prepare method is used to populate the VLAN
        # and Interfaces and finally the built resources are fetched using
        # {#to_puppet}
        #
        # There is a notion of an action to these methods, the action can be
        # either :add or :remove to facilate teardown
        class Rack < Base
          def portchannel_resource(number, fcoe=false, remove=false)
            raise("Managing port channels on TOR switches are not supported")
          end

          def mxl_interface_resource(interface, port_channel=nil)
            raise("Managing MXL interfaces for Uplinks on TOR switches are not supported")
          end

          # Creates interfaces resources for consumption by puppet
          #
          # @return [Hash]
          def to_puppet
            if port_resources["force10_interface"]
              port_resources["force10_interface"].keys.each do |interface|
                resource = port_resources["force10_interface"][interface]

                # the puppet type does not accept arrays for interfaces at the moment, so join them as csv and delete empty ones
                ["tagged_vlan", "untagged_vlan"].each do |prop|
                  next unless resource[prop].is_a?(Array)
                  resource[prop] = resource[prop].sort.uniq.join(",")
                end
              end
            end

            port_resources
          end

          # Prepares the final internal state for a given action
          #
          # This should be called before {#to_puppet} to construct
          # the state based on interface information created using
          # {#configure_interface_vlan}
          #
          # The return value indicates if there were any interfaces to
          # configure for the action, if it's false there's no point
          # in calling process for the type as there's nothing to process
          #
          # @param [:add, :remove] action
          # @return [Boolean] if any interfaces were found for the action
          def prepare(action)
            reset!
            validate_vlans!
            populate_force10_interface_resources(action)
            populate_vlan_resources(action)

            !port_resources.empty?
          end

          # Contrust asm::force10 resources for a VLAN
          #
          # Interfaces added with {#configure_interface_vlan} have various
          # associated VLANs, this adds resources to manage those VLANs
          # to the {#port_resources} hash
          #
          # @param number [String, Fixnum] vlan number
          # @param [Hash] properties
          # @option properties [String] :vlan_name
          # @option properties [String] :vlan_desc
          # @return [void]
          def vlan_resource(number, properties={})
            vlan = number.to_s

            port_resources["asm::force10"] ||= {}

            unless port_resources["asm::force10"][vlan]
              port_resources["asm::force10"][vlan] = {
                "vlan_name" => properties.fetch(:vlan_name, ""),
                "desc" => properties.fetch(:vlan_desc, "VLAN Created by ASM"),
                "before" => []
              }
            end

            if interfaces = interface_map.select {|i| i[:vlan] == vlan}
              interfaces.each do |interface|
                before_name = "Force10_interface[%s]" % interface[:interface]
                port_resources["asm::force10"][vlan]["before"] << before_name unless port_resources["asm::force10"][vlan]["before"].include?(before_name)
              end
            end
          end

          # Find all the interfaces for a certain action and create VLANs
          #
          # Interfaces are made using {#configure_interface_vlan} and are
          # marked as add or remove.  This finds all previously made interfaces
          # for a given action and calls {#vlan_resource) for each to construct
          # the correct asm::force10 resources
          #
          # @param [:add, :remove] action
          # @return [void]
          def populate_vlan_resources(action)
            action_interfaces = interface_map.select {|i| i[:action] == action}
            vlan_names = action_interfaces.map {|i| i[:vlan]}.sort.uniq

            vlan_names.each do |vlan|
              vlan_resource(vlan) unless action == :remove
            end
          end
        end
      end
    end
  end
end
