module ASM
  class Provider
    class Switch
      class Powerconnect
        # Manage Interface VLAN membership for Dell Powerconnect rack switches
        #
        # This class builds a temporary map of every interface and all its
        # VLAN memberships via {#configure_interface_vlan} and then turns that into
        # an appropriate puppet resource using {#to_puppet}
        class Rack
          attr_reader :logger
          attr_accessor :type, :provider

          def initialize(provider, resources={})
            @interfaces = []
            @resources = resources
            @provider = provider
            @logger = provider.logger
            @type = provider.type
          end

          def reset!
            @resources.clear
          end

          def interface_map
            @interfaces
          end

          def port_resources
            @resources
          end

          def prepare(action)
            reset!
            validate_vlans!
            populate_vlan_resources(action)
            !port_resources.empty?
          end

          def process!
            [:remove, :add].each do |action|
              super if prepare(action)
            end
          end

          # Configures an interface as part of a vlan
          #
          # @example configuring 2 interfaces
          #
          #    server.configure_interface_vlan("Te 1/1", "10", true)
          #    server.configure_interface_vlan("Te 1/1", "11", true)
          #    server.configure_interface_vlan("Te 1/1", "18", false)
          #
          #    rack.prepare(:add)
          #    rack.to_puppet
          #
          # The example above shows that all interface related VLANs get
          # created using {#configure_interface_vlan} - here 18 would be
          # untagged while the rest are tagged
          #
          # Once all the interfaces are known the {#prepare} and {#to_puppet}
          # calls will turn the list of interfaces into actual puppet resources
          #
          # @param interface [String] the interface name
          # @param vlan [String] the vlan to add the interface to
          # @param tagged [Boolean] true when its tagged
          # @param remove [Boolean] true if this is a teardown and needs to be removed
          # @return [void]
          def configure_interface_vlan(interface, vlan, tagged, remove=false)
            raise "Interface not specified for vlan %s" % vlan if interface.empty?
            action = remove ? :remove : :add

            interface_map << {
              :interface => interface.to_s,
              :vlan      => vlan.to_s,
              :tagged    => tagged,
              :action    => action
            }
          end

          # Validate the interface map for impossible configurations
          #
          # @return [void]
          # @raise [StandardError] for invalid config
          def validate_vlans!
            errors = 0

            interface_map.map {|i| i[:interface]}.sort.uniq.each do |interface|
              untagged = interface_map.count {|i| !i[:tagged] && i[:interface] == interface}

              if untagged > 1
                logger.warn("Attempt to configure %d untagged vlans on port %s" % [untagged, interface])
                errors += 1
              end
            end

            if errors > 0
              raise("Can only have one untagged network but found multiple untagged vlan requests for the same port on %s" % type.puppet_certname)
            end
          end

          # Find all the interfaces for a certain action and create VLANs
          #
          # Interfaces are made using {#configure_interface_vlan} and are
          # marked as add or remove.  This finds all previously made interfaces
          # for a given action and calls {#vlan_resource) for each to construct
          # the correct asm::powerconnect resources
          #
          # @param [:add, :remove] action
          # @return [void]
          def populate_vlan_resources(action)
            action_interfaces = interface_map.select {|i| i[:action] == action}

            action_interfaces.each do |iface|
              properties = {
                :tagged   => [],
                :untagged => []
              }
              action_interfaces.select {|i| i[:interface] == iface[:interface]}.each do |interface|
                if interface[:tagged]
                  properties[:tagged] << interface[:vlan]
                else
                  properties[:untagged] << interface[:vlan]
                end
              end

              interface_resource(iface[:interface], action, properties)
            end
          end

          # Construct powerconnect_interface and powerconnect_vlan resources for a VLAN
          #
          # Interfaces added with {#configure_interface_vlan} have various
          # associated VLANs, this adds resources to manage those VLANs
          # to the {#port_resources} hash
          #
          # @param interface [String] interface name
          # @param [:add, :remove] action
          # @param [Hash] properties
          # @option properties [Array[String]] :tagged
          # @option properties [Array[String]] :untagged
          # @return [void]
          def interface_resource(interface, action, properties={})
            interface = interface.to_s
            tagged = properties.fetch(:tagged, [])
            untagged = properties.fetch(:untagged, [])

            port_resources["powerconnect_interface"] ||= {}
            if action == :add
              port_resources["powerconnect_vlan"] ||= {}

              unless port_resources["powerconnect_interface"][interface]
                port_resources["powerconnect_interface"][interface] = {
                  "switchport_mode" => "general",
                  "portfast" => "true",
                  "shutdown" => "false",
                  "tagged_general_vlans" => tagged,
                  "untagged_general_vlans" => untagged
                }
              end
              vlans = tagged + untagged
              vlans.each do |vlan|
                next if port_resources["powerconnect_vlan"][vlan]
                port_resources["powerconnect_vlan"][vlan] = {
                  "ensure"  => "present",
                  "require" => "Powerconnect_interface[#{interface}]"
                }
              end
            elsif action == :remove
              resource = {
                interface => {
                  "remove_general_vlans" => tagged + untagged
                }
              }
              unless port_resources["powerconnect_interface"][interface]
                port_resources["powerconnect_interface"].merge!(resource)
              end
            end
          end

          # Creates interfaces resources for consumption by puppet
          #
          # @return [Hash]
          def to_puppet
            possible_resources = %w(powerconnect_vlan powerconnect_interface)
            possible_resources.each do |r_source|
              next unless port_resources[r_source]
              port_resources[r_source].keys.each do |name|
                resource = port_resources[r_source][name]
                resource.delete_if {|_k, v| v.empty?}
                resource.each do |k, v|
                  resource[k] = v.sort.uniq.join(",") if v.is_a? Array
                end
              end
            end

            port_resources
          end
        end
      end
    end
  end
end
