require "spec_helper"
require "asm/ipxe_builder"
require "asm/network_configuration"
require "asm/wsman"

describe ASM::IpxeBuilder do
  describe "render_bootstrap" do
    let(:nic_view_data) { SpecHelper.json_fixture("ipxe_builder/get_nic_view.json") }

    before(:each) do
      ASM::WsMan.stubs(:get_nic_view).returns(nic_view_data)
      ASM::WsMan.stubs(:get_bios_enumeration).returns([])
      ASM::Util.stubs(:get_preferred_ip).returns("172.25.3.100")
    end

    it "should render a static bootstrap script" do
      network_config_data = SpecHelper.json_fixture("ipxe_builder/static_pxe_network_configuration.json")
      network_config = ASM::NetworkConfiguration.new(network_config_data)
      network_config.add_nics!(:host => "host", :user => "user", :password => "password")
      expected = SpecHelper.load_fixture("ipxe_builder/static_bootstrap.ipxe")
      expect(ASM::IpxeBuilder.render_bootstrap(network_config, 4)).to eq(expected)
    end

    it "should render a dhcp bootstrap script" do
      network_config_data = SpecHelper.json_fixture("ipxe_builder/dhcp_pxe_network_configuration.json")
      network_config = ASM::NetworkConfiguration.new(network_config_data)
      network_config.add_nics!(:host => "host", :user => "user", :password => "password")
      expected = SpecHelper.load_fixture("ipxe_builder/dhcp_bootstrap.ipxe")
      expect(ASM::IpxeBuilder.render_bootstrap(network_config, 4)).to eq(expected)
    end
  end

  describe "generate_bootstrap" do
    it "should call render_bootstrap and write the file" do
      ASM::IpxeBuilder.expects(:render_bootstrap).with("network-config", 8).returns("script")
      File.expects(:write).with("/tmp/bootstrap.ipxe", "script")
      ASM::IpxeBuilder.generate_bootstrap("network-config", 8, "/tmp/bootstrap.ipxe")
    end
  end

  describe "build" do
    before(:each) do
      Tempfile.expects(:new).with("bootstrap.ipxe").returns(stub(:path => "/tmp/bootstrap.ipxe", :unlink => nil))
      ASM::IpxeBuilder.expects(:generate_bootstrap).with("rspec-network-config", 4, "/tmp/bootstrap.ipxe")
    end

    it "should generate bootstrap and make the ISO" do
      ASM::Util.expects(:run_command)
               .with("make", "-C", "/opt/src/ipxe/src", "bin/ipxe.iso", "EMBED=/tmp/bootstrap.ipxe")
               .returns(Hashie::Mash.new(:exit_status => 0, :stdout => "", :stderr => ""))
      FileUtils.expects(:mv).with("/opt/src/ipxe/src/bin/ipxe.iso", "/var/nfs/generated.iso")
      ASM::IpxeBuilder.build("rspec-network-config", 4, "/var/nfs/generated.iso")
    end

    it "should fail if the ISO build fails" do
      response = Hashie::Mash.new(:exit_status => 1, :stdout => "", :stderr => "")
      ASM::Util.expects(:run_command)
               .with("make", "-C", "/opt/src/ipxe/src", "bin/ipxe.iso", "EMBED=/tmp/bootstrap.ipxe")
               .returns(response)
      message = "iPXE ISO build failed: %s" % response.to_s
      expect {ASM::IpxeBuilder.build("rspec-network-config", 4, "/var/nfs/generated.iso")}.to raise_error(message)
    end
  end
end
