require 'spec_helper'
require 'asm/type'

ASM::Type.load_providers!

describe ASM::Provider::Switch::Brocade do
  let(:logger) { stub(:debug => nil, :warn => nil, :info => nil) }
  let(:raw_switches) { SpecHelper.json_fixture("switch_providers/switch_inventory.json") }

  let(:collection) { ASM::Service::SwitchCollection.new(logger) }
  let(:type) { collection.switch_by_certname("brocade_fos-172.17.9.15") }
  let(:provider) { type.provider }

  let(:service) { SpecHelper.service_from_fixture("switch_providers/deployment.json") }

  before(:each) do
    collection.stubs(:managed_inventory).returns(raw_switches)
    ASM::Service::SwitchCollection.stubs(:new).returns(collection)
  end

  describe "#active_fc_zone" do
    it "should find the correct active config" do
      expect(provider.active_fc_zone(nil)).to eq("Config_09_Top")
    end
  end

  describe "#fc_zones" do
    it "should support returning all known zones" do
      expect(provider.fc_zones).to eq(["ASM_GP181Y1", "ASM_H4N71Y1", "ASM_HV7QQV1", "ASM_Host_Top", "Cntrl_Phys_Top", "Cntrl_Virtual_Top", "ESXi_9_3"])
    end

    it "should support limiting zones based on wwpn membership" do
      provider.facts["Zone_Members"] = {"ASM_GP181Y1" => ["50:00:d3:10:00:55:5b:3d"], "RSPEC" => ["50:00:d3:10:00:55:5b:3d"]}
      expect(provider.fc_zones("50:00:d3:10:00:55:5b:3d")).to eq(["ASM_GP181Y1", "RSPEC"])
      expect(provider.fc_zones("50:00:D3:10:00:55:5B:3D")).to eq(["ASM_GP181Y1", "RSPEC"])
    end
  end

  describe "#find_mac" do
    it "should return the interface when found" do
      ["21:00:00:24:ff:46:63:5a", "21:00:00:24:ff:4a:70:3e", "20:11:00:05:33:45:75:f3"].each do |mac|
        expect(provider.find_mac(mac)).to eq("port0")
      end

      expect(provider.find_mac("20:12:00:05:33:45:75:f3")).to eq("port1")
    end

    it "should return nil when there are no facts" do
      provider.facts = nil
      expect(provider.find_mac("rspec:rspec")).to be(nil)

      provider.facts = {}
      expect(provider.find_mac("rspec:rspec")).to be(nil)
    end
  end

  describe "#handles_switch?" do
    it "should handle switches starting with brocade_fos/ and not others" do
      expect(provider.class.handles_switch?("refId" => "brocade_fos_rspec")).to be(true)
      expect(provider.class.handles_switch?("refId" => "dell_rspec")).to be(false)
    end
  end
end
