require 'spec_helper'
require 'asm/provider/switch/powerconnect/rack'

describe ASM::Provider::Switch::Powerconnect::Rack do
  let(:logger) { stub(:debug => nil, :warn => nil, :info => nil) }
  let(:provider) { stub(:type => stub(:puppet_certname => "rspec"), :logger => logger) }
  let(:server) { ASM::Provider::Switch::Powerconnect::Rack.new(provider) }

  describe "#vlaidate_vlans!" do
    it "should allow multiple untagged vlans on different ports" do
      server.configure_interface_vlan("18", "1", false)
      server.configure_interface_vlan("20", "1", false)

      server.validate_vlans!
    end

    it "should not allow multiple untagged vlans on the same port" do
      server.configure_interface_vlan("1", "1", false)
      server.configure_interface_vlan("1", "2", false)

      expect {
        server.validate_vlans!
      }.to raise_error("Can only have one untagged network but found multiple untagged vlan requests for the same port on rspec")
    end
  end

  describe "#configure_interface_vlan" do
    it "should add the interface correctly" do
      server.configure_interface_vlan("Eth1/21", "28", false)

      expect(server.interface_map).to include({:interface => "Eth1/21", :vlan => "28", :tagged => false, :action => :add})
    end
  end

  describe "#populate_vlan_resources" do
    context "when action is :add" do
      it "should create the correct vlan resources" do
        server.configure_interface_vlan("Eth1/21", "28", false)

        server.expects(:interface_resource).with("Eth1/21", :add, {:tagged => [], :untagged => ["28"]})
        server.populate_vlan_resources(:add)
      end
    end

    context "when action is :remove" do
      it "should create vlan resources with remove" do
        server.configure_interface_vlan("Eth1/21", "28", false, true)

        server.expects(:interface_resource).with("Eth1/21", :remove, {:tagged => [], :untagged => ["28"]})
        server.populate_vlan_resources(:remove)
      end
    end
  end

  describe "#interface_resource" do
    context "when untagged" do
      it "should create a the correct resources" do
        server.interface_resource("Eth1/21", :add, {:tagged => [], :untagged => ["28"]})

        expect(server.port_resources["powerconnect_interface"]).to include("Eth1/21")
        interface = server.port_resources["powerconnect_interface"]["Eth1/21"]
        expect(interface["untagged_general_vlans"]).to eq(["28"])
        expect(interface["portfast"]).to eq("true")

        expect(server.port_resources["powerconnect_vlan"]).to include("28")
        vlan = server.port_resources["powerconnect_vlan"]["28"]
        expect(vlan["ensure"]).to eq("present")
        expect(vlan["require"]).to eq("Powerconnect_interface[Eth1/21]")
      end
    end

    context "when remove" do
      it "should create the correct teardown resource" do
        server.interface_resource("Eth1/21", :remove, {:tagged => [], :untagged => ["28"]})

        expect(server.port_resources["powerconnect_interface"]).to include("Eth1/21")
        interface = server.port_resources["powerconnect_interface"]["Eth1/21"]
        expect(interface["remove_general_vlans"]).to eq(["28"])
      end
    end
  end

  describe "#to_puppet" do
    it "should join arry properties to strings" do
      server.stubs(:port_resources).returns({
                                              "powerconnect_interface" => {
                                                  "Eth1/21" => {
                                                      "untagged_general_vlans" => ["28"],
                                                      "tagged_general_vlans" => []
                                                  }
                                              }
                                          })

      expect(server.to_puppet).to eq({
                                       "powerconnect_interface" => {
                                           "Eth1/21" => {
                                               "untagged_general_vlans" => "28",
                                           }
                                       }
                                   })
    end
  end

  describe "#prepare" do
    it "should correctly configure the internal state" do
      prepare = sequence(:prepare)
      server.expects(:reset!).in_sequence(prepare)
      server.expects(:validate_vlans!).in_sequence(prepare)
      server.expects(:populate_vlan_resources).with(:add).in_sequence(prepare)

      server.stubs(:port_resources).returns({})

      expect(server.prepare(:add)).to be(false)
    end

    it "should correctly indicate if a process! is needed" do
      server.stubs(:reset!)
      server.stubs(:populate_vlan_resources)

      server.stubs(:port_resources).returns({})
      expect(server.prepare(:add)).to be(false)

      server.stubs(:port_resources).returns({"powerconnect_interface" => {}})
      expect(server.prepare(:add)).to be(true)
    end
  end

  describe "#reset!" do
    it "should clear the prepared resources" do
      server.port_resources["rspec"] = true

      expect(server.port_resources).to_not be_empty
      server.reset!
      expect(server.port_resources).to be_empty
    end
  end

end