require "spec_helper"
require "asm/type"

ASM::Type.load_providers!

describe ASM::Type::Server do
  let(:raw_switches) do
    SpecHelper.json_fixture("switch_providers/switch_inventory.json")
  end

  let(:logger) { stub(:debug => nil, :warn => nil, :info => nil) }

  let(:collection) { ASM::Service::SwitchCollection.new(logger) }
  let(:switch_type) { collection.switches.first }

  let(:service) { SpecHelper.service_from_fixture("Teardown_EQL_VMware_Cluster.json", logger) }
  let(:db) { stub }
  let(:deployment) { stub(:id => "1234", :db => db, :debug? => false, :logger => logger) }
  let(:server_component) { service.components_by_type("SERVER").first }

  let(:provider) { stub(:hostname => "rspec.host", :policy_name => "policy-rspec.host-1234") }
  let(:type) { server_component.to_resource(deployment, logger) }
  let(:raw_network_params) { server_component.configuration["asm::esxiscsiconfig"]["bladeserver-15kvd42"] }

  before(:each) do
    collection.stubs(:managed_inventory).returns(raw_switches)
    type.stubs(:provider).returns(provider)
    type.stubs(:dell_server?).returns(true)
    ASM::NetworkConfiguration.any_instance.stubs(:add_nics!)
  end

  describe "#fcoe_wwpns" do
    it "should return [] when there are no fcoe san networks" do
      ASM::WsMan.expects(:get_fcoe_wwpn).never
      expect(type.fcoe_wwpns).to eq([])
    end

    it "should return the correct wwpns" do
      type.expects(:fcoe_san_partitions).returns([
        {"id" => "373ED367-028D-4DF7-8176-0DF26C263963", "fqdd" => "NIC.Integrated.1-1-4"},
        {"id" => "39314B90-BDC0-4093-81E5-9A9A121ED8D1", "fqdd" => "NIC.Integrated.1-2-4"}
      ])

      ASM::WsMan.expects(:get_fcoe_wwpn).returns({
        "NIC.Integrated.1-1-4" => {"fcoe_offload_mode"=>"2", "fcoe_wwnn"=>"00:00:00:00:00:00", "fcoe_permanent_fcoe_macaddress"=>"00:00:00:00:00:00", "virt_wwn"=>"20:00:74:86:7A:EF:48:3D", "virt_wwpn"=>"20:01:74:86:7A:EF:48:3D", "wwn"=>"20:00:74:86:7A:EF:48:3D", "wwpn"=>"20:01:74:86:7A:EF:48:3D"},
        "NIC.Integrated.1-2-4" => {"fcoe_offload_mode"=>"2", "fcoe_wwnn"=>"00:00:00:00:00:00", "fcoe_permanent_fcoe_macaddress"=>"00:00:00:00:00:00", "virt_wwn"=>"20:00:74:86:7A:EF:48:3F", "virt_wwpn"=>"20:01:74:86:7A:EF:48:3F", "wwn"=>"20:00:74:86:7A:EF:48:3F", "wwpn"=>"20:01:74:86:7A:EF:48:3F"},
        "NIC.Integrated.1-1-1" => {"fcoe_offload_mode"=>"3", "fcoe_wwnn"=>"00:00:00:00:00:00", "fcoe_permanent_fcoe_macaddress"=>"00:00:00:00:00:00", "virt_wwn"=>"20:00:74:86:7A:EF:48:31", "virt_wwpn"=>"20:01:74:86:7A:EF:48:31", "wwn"=>"20:00:74:86:7A:EF:48:31", "wwpn"=>"20:01:74:86:7A:EF:48:31"}
      })

      expect(type.fcoe_wwpns).to eq(["20:01:74:86:7A:EF:48:3D", "20:01:74:86:7A:EF:48:3F"])
    end
  end

  describe "#fc_wwpns" do
    it "should retrieve the wwpns from wsman" do
      provider.expects(:fc_wwpns).returns(["21:00:00:24:FF:46:63:5A", "21:00:00:24:FF:46:63:5B"])
      expect(type.fc_wwpns).to eq(["21:00:00:24:FF:46:63:5A", "21:00:00:24:FF:46:63:5B"])
    end
  end

  describe "#fc?" do
    it "should detect fc enabled volumes" do
      type.expects(:related_volumes).returns([stub(:fc? => true), stub(:fc? => false)])
      expect(type.fc?).to be(true)

      type.expects(:related_volumes).returns([stub(:fc? => false), stub(:fc? => false)])
      expect(type.fc?).to be(false)
    end
  end

  describe "#is_hypervisor?" do
    it "sould check the current os" do
      provider.expects(:os_image_type).returns("rspec")
      type.expects(:is_hypervisor_os?).with("rspec").returns(true)

      expect(type.is_hypervisor?).to be(true)
    end
  end

  describe "#is_hypervisor_os?" do
    it "should support the known good OSes" do
      ["hyperv", "HyPerV", "vmware_esxi", "vmware_ESXI"].each do |os|
        expect(type.is_hypervisor_os?(os)).to be(true)
      end
    end

    it "should not support unknown OSes" do
      expect(type.is_hypervisor_os?("rspec")).to be(false)
    end
  end

  describe "#configure_networking!" do
    before(:each) do
      type.stubs(:management_ip).returns("172.14.3.4")
    end
    it "should not fail when no switches are found" do
      type.stubs(:related_switches).returns([])
      logger.expects(:warn).with("Could not find any switches to configure for server bladeserver-15kvd42")
      ASM::Type::Switch.any_instance.expects(:configure_server).never
      type.configure_networking!
    end

    it "should configure the server on related switches" do
      switches = [stub(:puppet_certname => "switch1"), stub(:puppet_certname => "switch2")]

      type.stubs(:related_switches).returns(switches)
      switches.each do |s|
        s.stubs(:managed?).returns(true)
        s.stubs(:model).returns("MXL")
        s.stubs(:management_ip).returns("10.100.2.1")
      end

      switches[0].expects(:configure_server).with(type, false)
      switches[1].expects(:configure_server).with(type, false)
      type.expects(:db_log).twice

      type.configure_networking!
    end

    it "should configure the server for only one switch" do
      selected_switch = stub(:puppet_certname => "switch")
      selected_switch.stubs(:managed?).returns(true)
      switches = [selected_switch, stub(:puppet_certname => "switch2"), stub(:puppet_certname => "switch3")]
      type.stubs(:related_switches).returns(switches)
      switches.each do |s|
        s.stubs(:managed?).returns(true)
        s.stubs(:model).returns("MXL")
        s.stubs(:management_ip).returns("10.100.2.1")
      end
      switches[0].expects(:configure_server).never
      switches[1].expects(:configure_server).never
      selected_switch.expects(:configure_server).with(type, false)
      type.expects(:db_log)
      type.configure_networking!(false, :switch => selected_switch)
    end

    it "should not configure an unrelated switch" do
      selected_switch = stub(:puppet_certname => "switch")
      selected_switch.stubs(:managed?).returns(true)
      switches = [stub(:puppet_certname => "switch1"), stub(:puppet_certname => "switch2")]
      type.stubs(:related_switches).returns(switches)
      expect do
        type.configure_networking!(false, :switch => selected_switch)
      end.to raise_error("Switch switch not connected to bladeserver-15kvd42")
    end

    it "should display the correct logging message" do
      switch = stub(:puppet_certname => "switch", :supports_resource? => true)
      switch.stubs(:model).returns("MXL")
      switch.stubs(:management_ip).returns("10.100.2.1")
      switch.stubs(:managed?).returns(true)

      type.stubs(:related_switches).returns([switch])
      type.expects(:db_log).with(:info, "Configuring server 15KVD42 172.14.3.4 networking on MXL 10.100.2.1")
      switch.expects(:configure_server).with(type, false)
      type.configure_networking!(false)
    end
  end

  describe "#powered_on?" do
    it "should return the correct on state" do
      type.expects(:power_state).returns(:on)
      expect(type.powered_on?).to be(true)

      type.expects(:power_state).returns(:off)
      expect(type.powered_on?).to be(false)
    end
  end

  describe "#network_interfaces" do
    before(:each) do
      network_config = ASM::NetworkConfiguration.new(
        SpecHelper.json_fixture("switch_providers/bladeserver-gp181y1_network_config.json")
      )
      type.stubs(:network_config).returns(network_config)
    end

    it "should return all interfaces by default" do
      interfaces = type.network_interfaces

      expect(interfaces.size).to be(2)

      expect(interfaces.map{|i| i.partitions.first.fqdd}).to eq(["NIC.Integrated.1-1-1", "NIC.Integrated.1-2-1"])
    end
  end

  describe "#configured_interfaces" do
    before(:each) do
      network_config = ASM::NetworkConfiguration.new(
          SpecHelper.json_fixture("switch_providers/bladeserver-gp181y1_network_config.json")
      )
      network_config.cards.first.interfaces.first.partitions.each do |partition|
        partition.networkObjects = []
      end
      type.stubs(:network_config).returns(network_config)
    end

    it "should skip interfaces without networks" do
      interfaces = type.configured_interfaces
      expect(interfaces.size).to be(1)
      expect(interfaces.map { |i| i.partitions.first.fqdd }).to eq(["NIC.Integrated.1-2-1"])
    end
  end

  describe "#switches" do
    before(:each) do
      ASM::Service::SwitchCollection.any_instance.stubs(:managed_inventory).returns(raw_switches)
      ASM::Service::SwitchCollection.any_instance.stubs(:deployment).returns(deployment)

      ["dell_iom-172.17.9.174", "dell_ftos-172.17.9.13", "dell_ftos-172.17.9.14", "dell_iom-172.17.9.171", "brocade_fos-172.17.9.15", "brocade_fos-172.17.9.16"].each do |sw|
        facts_fixture = "switch_providers/%s_facts.json" % sw
        ASM::PrivateUtil.stubs(:facts_find).with(sw).returns(
          JSON.parse(SpecHelper.load_fixture(facts_fixture))
        )
      end
      ASM::PrivateUtil.stubs(:facts_find).with("bladeserver-15kvd42").returns({})
      ASM::PrivateUtil.stubs(:facts_find).with("bladeserver-gp181y1").returns({})

      network_config = ASM::NetworkConfiguration.new(
        SpecHelper.json_fixture("switch_providers/bladeserver-gp181y1_network_config.json")
      )
      type.stubs(:network_config).returns(network_config)
      type.stubs(:save_facts!)
      type.stubs(:fc_interfaces).returns([])
    end

    it "should correctly identify connected switches" do
      interfaces = type.network_interfaces

      expect(type.related_switches[0]).to be_a(ASM::Type::Switch)
      expect(type.related_switches[0].provider_name).to eq("force10")
      expect(type.related_switches[0].puppet_certname).to eq("dell_iom-172.17.9.171")
      expect(type.related_switches[0].find_mac(interfaces[0].partitions.first["mac_address"])).to eq("Te 0/2")

      expect(type.related_switches[1]).to be_a(ASM::Type::Switch)
      expect(type.related_switches[1].provider_name).to eq("force10")
      expect(type.related_switches[1].puppet_certname).to eq("dell_iom-172.17.9.174")
      expect(type.related_switches[1].find_mac(interfaces[1].partitions.first["mac_address"])).to eq("Te 0/2")
    end
  end

  describe "#network_topology_cache" do
    let(:switch1) { mock(:puppet_certname => "dell_iom-172.17.9.171")}
    let(:switch2) { mock(:puppet_certname => "dell_iom-172.17.9.174")}

    before(:each) do
      facts = {"network_topology" => [["e0:db:55:22:f9:a0", "dell_iom-172.17.9.171", "Te 0/2"],
                                      ["e0:db:55:22:f9:a2", "dell_iom-172.17.9.174", "Te 0/3"]].to_json}
      type.stubs(:retrieve_facts).returns(facts)
    end

    it "should create a hash of macs to [switch, port] and add relations" do
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with("dell_iom-172.17.9.171").returns(switch1).twice
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with("dell_iom-172.17.9.174").returns(switch2).twice
      type.expects(:add_relation).with(switch1)
      switch1.expects(:add_relation).with(type)
      type.expects(:add_relation).with(switch2)
      switch2.expects(:add_relation).with(type)
      expect(type.network_topology_cache).to eq({"e0:db:55:22:f9:a0" => [switch1.puppet_certname, "Te 0/2"],
                                                 "e0:db:55:22:f9:a2" => [switch2.puppet_certname, "Te 0/3"]})
    end

    it "should reject switches not in switch_collection" do
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with("dell_iom-172.17.9.171").returns(switch1).twice
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with("dell_iom-172.17.9.174").returns(nil)
      type.expects(:add_relation).with(switch1)
      switch1.expects(:add_relation).with(type)
      expect(type.network_topology_cache).to eq({"e0:db:55:22:f9:a0" => [switch1.puppet_certname, "Te 0/2"]})
    end
  end

  describe "#add_network_topology_cache" do
    let(:cache) { {} }
    let(:switch1) { stub(:puppet_certname => "dell_iom-172.17.9.171") }

    before(:each) do
      type.stubs(:network_topology_cache).returns(cache)
      type.expects(:add_relation).with(switch1)
      switch1.expects(:add_relation).with(type)
    end

    it "should add relation between server and switch" do
      type.add_network_topology_cache("rspec-mac", switch1, "rspec-port")
      expect(cache).to eq({"rspec-mac" => [switch1.puppet_certname, "rspec-port"]})
    end

    it "should downcase the mac address" do
      type.add_network_topology_cache("RSPEC-MAC", switch1, "rspec-port")
      expect(cache).to eq({"rspec-mac" => [switch1.puppet_certname, "rspec-port"]})
    end
  end

  describe "#network_topology" do
    let(:network_config) do
      ASM::NetworkConfiguration.new(
          SpecHelper.json_fixture("switch_providers/bladeserver-gp181y1_network_config.json")
      )
    end

    let(:fc_interfaces) { SpecHelper.json_fixture("switch_providers/bladeserver-gp181y1_fc_interfaces.json") }

    let(:interface1) { network_config.cards[0].interfaces[0] }
    let(:interface2) { network_config.cards[0].interfaces[1] }
    let(:mac1) { interface1.partitions.first.mac_address }
    let(:mac2) { interface2.partitions.first.mac_address }
    let(:mac3) { fc_interfaces[0]["wwpn"] }
    let(:mac4) { fc_interfaces[1]["wwpn"] }
    let(:switch1) { stub(:puppet_certname => "rspec-switch1")}
    let(:switch2) { stub(:puppet_certname => "rspec-switch2")}
    let(:switch3) { stub(:puppet_certname => "rspec-fcswitch1")}
    let(:switch4) { stub(:puppet_certname => "rspec-fcswitch2")}

    before(:each) do
      type.stubs(:network_config).returns(network_config)
      provider.stubs(:fc_interfaces).returns(fc_interfaces.map{|f| Hashie::Mash.new(f)})
    end

    it "should retrieve connectivity and cache them" do
      ASM::Service::SwitchCollection.any_instance.expects(:switch_port_for_mac)
          .with(mac1).returns([switch1, "Te 0/2"])
      ASM::Service::SwitchCollection.any_instance.expects(:switch_port_for_mac)
          .with(mac2).returns([switch2, "Te 0/3"])
      ASM::Service::SwitchCollection.any_instance.expects(:switch_port_for_mac)
          .with(mac3).returns([switch3, "port1"])
      ASM::Service::SwitchCollection.any_instance.expects(:switch_port_for_mac)
          .with(mac4).returns([switch4, "port2"])
      type.stubs(:network_topology_cache).returns({})
      type.expects(:add_network_topology_cache).with(mac1, switch1, "Te 0/2")
      type.expects(:add_network_topology_cache).with(mac2, switch2, "Te 0/3")
      type.expects(:add_network_topology_cache).with(mac3, switch3, "port1")
      type.expects(:add_network_topology_cache).with(mac4, switch4, "port2")
      type.expects(:save_facts!)

      expect(type.network_topology).to eq([{:interface => interface1,
                                            :interface_type => "ethernet",
                                            :switch => switch1,
                                            :port => "Te 0/2"},
                                           {:interface => interface2,
                                            :interface_type => "ethernet",
                                            :switch => switch2,
                                            :port => "Te 0/3"},
                                           {:interface => fc_interfaces[0],
                                            :interface_type => "fc",
                                            :switch => switch3,
                                            :port => "port1"},
                                           {:interface => fc_interfaces[1],
                                            :interface_type => "fc",
                                            :switch => switch4,
                                            :port => "port2"} ])
    end

    it "should retrieve cached connectivity" do
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with(switch1.puppet_certname).returns(switch1)
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with(switch2.puppet_certname).returns(switch2)
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with(switch3.puppet_certname).returns(switch3)
      ASM::Service::SwitchCollection.any_instance.expects(:switch_by_certname)
          .with(switch4.puppet_certname).returns(switch4)
      type.stubs(:network_topology_cache).returns({mac1.downcase => [switch1.puppet_certname, "Te 0/2"],
                                                   mac2.downcase => [switch2.puppet_certname, "Te 0/3"],
                                                   mac3.downcase => [switch3.puppet_certname, "port1"],
                                                   mac4.downcase => [switch4.puppet_certname, "port2"]})
      expect(type.network_topology).to eq([{:interface => interface1,
                                            :interface_type => "ethernet",
                                            :switch => switch1,
                                            :port => "Te 0/2"},
                                           {:interface => interface2,
                                            :interface_type => "ethernet",
                                            :switch => switch2,
                                            :port => "Te 0/3"},
                                           {:interface => fc_interfaces[0],
                                            :interface_type => "fc",
                                            :switch => switch3,
                                            :port => "port1"},
                                           {:interface => fc_interfaces[1],
                                            :interface_type => "fc",
                                            :switch => switch4,
                                            :port => "port2"} ])
    end
  end

  describe "#missing_network_topology" do
    let(:network_config) do
      ASM::NetworkConfiguration.new(
          SpecHelper.json_fixture("switch_providers/bladeserver-gp181y1_network_config.json")
      )
    end
    let(:interface1) { network_config.cards[0].interfaces[0] }
    let(:interface2) { network_config.cards[0].interfaces[1] }
    let(:mac1) { interface1.partitions.first.mac_address }
    let(:mac2) { interface2.partitions.first.mac_address }
    let(:switch1) { mock("rspec-switch1")}

    before(:each) do
      type.stubs(:network_config).returns(network_config)
      provider.stubs(:fc_interfaces).returns([])
      ASM::Service::SwitchCollection.any_instance.expects(:switch_port_for_mac)
          .with(mac1).returns([switch1, "Te 0/2"])
      ASM::Service::SwitchCollection.any_instance.expects(:switch_port_for_mac)
          .with(mac2).returns(nil)
      type.stubs(:network_topology_cache).returns({})
      type.expects(:add_network_topology_cache).with(mac1, switch1, "Te 0/2")
      type.expects(:save_facts!)
    end


    it "should find interfaces without switch ports" do
      expect(type.missing_network_topology).to eq([interface2.partitions.first, ])
    end

    it "should skip unconfigured interfaces" do
      type.expects(:configured_interface?).with(interface2).returns(false)
      expect(type.missing_network_topology).to eq([])
    end
  end

  describe "#deployment_completed?" do
    it "should return true if the db says its done" do
      type.expects(:db_execution_status).returns("complete")
      expect(type.deployment_completed?).to be(true)
    end

    it "should return if not completed in the db and its a bfs machine" do
      type.expects(:db_execution_status).returns(nil)
      type.expects(:boot_from_san?).returns(true)
      type.expects(:razor_status).never
      expect(type.deployment_completed?).to be(false)
    end

    it "should check razor otherwise" do
      type.stubs(:db_execution_status).returns(nil)
      type.stubs(:boot_from_san?).returns(false)
      type.stubs(:has_os?).returns(true)
      type.stubs(:os_image_type).returns("redhat7")

      type.expects(:razor_status).returns({:status=> :rspec})
      expect(type.deployment_completed?).to be(false)

      type.expects(:razor_status).returns({:status => :boot_local})
      expect(type.deployment_completed?).to be(true)

      type.expects(:razor_status).returns(:status => :boot_local_2)
      expect(type.deployment_completed?).to be(true)
    end
  end

  describe "#db_execution_status" do
    it "should return the found status otherwise nil" do
      db.expects(:get_component_status).with("B0151383-9478-459F-A420-CAED5DE5B063").returns({:status => "rspec"})
      expect(type.db_execution_status).to eq("rspec")

      db.expects(:get_component_status).with("B0151383-9478-459F-A420-CAED5DE5B063").returns(nil)
      expect(type.db_execution_status).to eq(nil)
    end
  end

  describe "#razor_status" do
    it "should report the razor status when the node is found" do
      ASM::Razor.expects(:new).returns(razor = stub)
      razor.expects(:find_node).with("15KVD42").returns({"name" => "rspec"})
      razor.expects(:task_status).with("rspec", "policy-rspec.host-1234").returns({:status => "rspec"})

      expect(type.razor_status).to eq({:status => "rspec"})
    end
  end

  describe "#rackserver?" do
    it "should detect rack servers correctly" do
      type.expects(:physical_type).returns("RACK")
      expect(type.rackserver?).to be(true)

      type.expects(:physical_type).returns("BLADE")
      expect(type.rackserver?).to be(false)
    end
  end
  describe "#fcoe?" do
    it "should report false when there are no fcoe networks" do
      type.expects(:fcoe_san_networks).returns({})
      expect(type.fcoe?).to be(false)
    end

    it "should report false when there are no fcoe networks" do
      type.expects(:fcoe_san_networks).returns({1=>2})
      expect(type.fcoe?).to be(true)
    end
  end

  describe "#fcoe_san_networks" do
    it "should retrieve the STORAGE_FCOE_SAN networks" do
      type.expects(:network_config).returns(nc = stub)
      nc.expects(:get_networks).with("STORAGE_FCOE_SAN").returns({"rspec" => 1})
      expect(type.fcoe_san_networks).to eq({"rspec" => 1})
    end
  end

  describe "#fabric_info" do
    let(:rack_network_config_json) { SpecHelper.json_fixture("network_configuration/rack_partitioned.json") }
    let(:rack_network_config) { ASM::NetworkConfiguration.new(rack_network_config_json) }

    it "should create correct rack server info based on cards" do
      type.stubs(:network_config).returns(rack_network_config)
      expect(type.fabric_info).to eq({"Fabric A"=>2})
    end
  end

  describe "#network_params" do
    it "should fetch asm::esxiscsiconfig from the component" do
      expect(type.network_params).to eq(raw_network_params)
    end

    it "should return an empty hash when none is found" do
      server_component.expects(:resource_by_id).with("asm::esxiscsiconfig").returns(nil)
      expect(type.network_params).to eq({})
    end
  end

  describe "#network_config" do
    let(:config) { mock }

    before(:each) do
      ASM::NetworkConfiguration.stubs(:new).with(raw_network_params["network_configuration"]).returns(config)
      type.stubs(:device_config).returns({"rspec" => "device"})
    end

    it "should create instances of ASM::NetworkConfiguration" do
      config.expects(:add_nics!).with({"rspec" => "device"}, {:add_partitions => true})
      type.network_config
    end
  end

  describe "#management_network" do
    it "should retrieve the management network from the network config" do
      config = ASM::NetworkConfiguration.new(raw_network_params["network_configuration"])

      expect(type.management_network).to eq(config.get_network('HYPERVISOR_MANAGEMENT'))
      expect(type.management_network.name).to eq("HypervisorManagement")
      expect(type.management_network.id).to eq("ff8080814dbf2d1d014dc298d147006f")
      expect(type.management_network.staticNetworkConfiguration["ipAddress"]).to eq("172.28.10.63")
    end
  end

  describe "#static_network_config" do
    it "should retrieve the management network static config" do
      expect(type.static_network_config).to eq(type.management_network["staticNetworkConfiguration"])
    end
  end

  describe "#primary_dnsserver" do
    it "should retrieve the primaryDns" do
      expect(type.primary_dnsserver).to eq("172.20.0.8")
    end
  end

  describe "#secondary_dnsserver" do
    it "should retrieve the secondaryDns" do
      type.stubs(:static_network_config).returns("secondaryDns" => "8.8.8.8")
      expect(type.secondary_dnsserver).to eq("8.8.8.8")
    end
  end

  describe "#hostip" do
    it "should supply the hosts ip in the static network" do
      expect(type.hostip).to eq(type.static_network_config["ipAddress"])
    end
  end

  describe "#hostname" do
    it "should get the hostname from the provider" do
      type.expects(:provider).returns(mock(:hostname))
      type.hostname
    end
  end

  describe "#hyperv_config" do
    it "should return the data from the provider" do
      provider.stubs(:to_hash).with(true, :hyperv).returns({"rspec" => true})
      expect(type.hyperv_config).to eq({"rspec" => true})
    end
  end

  [:delete_server_cert!, :leave_cluster!, :clean_related_volumes!, :clean_virtual_identities!, :power_off!, :delete_server_node_data!].each do |m|
    describe "#%s" % m do
      it "should delegate to the provider" do
        type.expects(:delegate).with(provider, m).once
        type.send(m)
      end
    end
  end

  describe "#interface_in_vlan?" do
    context "when vlan is found" do
      it "should return true" do
        port_interface = "Eth1/21"
        facts = SpecHelper.json_fixture("switch_providers/vlan_information.json")
        switch_type.stubs(:facts).returns(facts)
        expect(type.interface_in_vlan?(switch_type, port_interface, "50", false)).to eq(true)
      end
    end

    context "when vlan is not found" do
      it "should return false" do
        port_interface = "Eth1/4"
        facts = SpecHelper.json_fixture("switch_providers/vlan_information.json")
        switch_type.stubs(:facts).returns(facts)
        expect(type.interface_in_vlan?(switch_type, port_interface,"50", true)).to eq(false)
      end
    end
  end

  describe "#parse_interface" do
    it "should parse group without range or letters or stack" do
      pg = "3"
      expect(type.parse_interface(pg)).to eq([[],"3",""])
    end
    it "should parse with range" do
      pg = "1-12"
      expect(type.parse_interface(pg)).to eq([[],"1-12",""])
    end
    it "should parse with single stack and range" do
      pg = "1/2-3"
      expect(type.parse_interface(pg)).to eq([["1"],"2-3","1"])
    end
    it "should parse with double stack and range" do
      pg = "1/2/3-4"
      expect(type.parse_interface(pg)).to eq([["1", "2"],"3-4","1/2"])
    end
    it "should parse with double stack and no range" do
      pg = "1/2/4"
      expect(type.parse_interface(pg)).to eq([["1", "2"],"4","1/2"])
    end
    it "should parse with single stack, letters, speed" do
      pg = "Te1/2-3"
      expect(type.parse_interface(pg)).to eq([["1"],"2-3","1"])
    end
    it "should parse with double stack, range, speed" do
      pg = "Po1/2/3-4"
      expect(type.parse_interface(pg)).to eq([["1","2"],"3-4","1/2"])
    end
  end

  describe "#validate_networking!" do
    it "should return true when switch configuration is correct" do
      switch_type.expects(:validate_network_config).with(type).returns(true)
      expect(type.validate_networking!(switch_type)).to be(true)
    end

    it "should returns false when switch configuration is incorrect" do
      switch_type.expects(:validate_network_config).with(type).returns(false)
      expect(type.validate_networking!(switch_type)).to be(false)
    end

  end
end
